@@ -4,19 +4,19 @@ use strict;
 use warnings;
 
 use lib '[% demo_root %]/lib';
-use Apache2 ();
-use Apache::Const;
+use Apache ();
 use Apache::Request ();
 use Apache::Cookie ();
+use Apache::Constants qw(:response);
 use Apache::Util ();
 use Apache::Status ();
 
 use DBI; 
-DBI->install_driver('SQLite2');
-
+DBI->install_driver('SQLite');
 use Class::DBI ();
-use Class::DBI::Factory ();
 $Class::DBI::Weaken_Is_Available = 0;	#disables unique-object stash for now
+
+use Class::DBI::Factory ();
 Class::DBI::Factory->add_status_menu;
 
 use Template ();
