@@ -1,11 +1,9 @@
 package Class::DBI::Factory::Handler;
 use strict;
 
-use Apache2;
-use Apache::Const qw(:common);
+use Apache::Constants qw(:response);
 use Apache::Request ();
 use Apache::Cookie ();
-use Apache::Upload ();
 
 use IO::File;
 use Carp ();
@@ -14,9 +12,9 @@ use List::Util qw(first max min);
 use Data::Dumper;
 
 require 5.6.0;
-use vars qw( $VERSION );
+use vars qw( $VERSION $factory_class );
 
-$VERSION = '0.9';      # mod_perl 2 only. well, let's see.
+$VERSION = '0.94';      # mod_perl 1 only. see Handler2 for a mod_perl2 version.
 $|++;
 
 =head1 NAME
@@ -103,20 +101,20 @@ There are several ways to make use of this. You can use it exactly as it is, to
 sub build_page {
 	my $self = shift;
 	
-	$self->debug(1, "\n\n\n____________REQUEST: " . $self->full_url);
-    $self->debug(3, "task sequence: " . join(', ', $self->task_sequence));
+	$self->debug(1, "\n\n\n____________REQUEST: " . $self->full_url, 'handler');
+    $self->debug(3, "task sequence: " . join(', ', $self->task_sequence), 'handler');
     my $return_code = OK;
 	
     try {
         for($self->task_sequence) {
-            $self->debug(2, "\n\n____________${_}");
+            $self->debug(2, "\n\n____________${_}", 'handler');
             $self->$_();
         }
     }
     
     catch Exception::OK with {
         my $x = shift;
-        $self->debug(1, 'caught OK exception: ' . $x->text);
+        $self->debug(1, 'caught OK exception: ' . $x->text, 'handler');
         $self->view( $x->view );
         $self->error( @{ $x->errors }) if $x->errors;
         $self->message( $x->text );
@@ -124,19 +122,19 @@ sub build_page {
     }
     catch Exception::NOT_FOUND with {
         my $x = shift;
-        $self->debug(1, 'caught NOT_FOUND exception: ' . $x->text);
+        $self->debug(1, 'caught NOT_FOUND exception: ' . $x->text, 'handler');
         my $view = $x->view || 'notfound';
         $self->return_error( $view, $x );
     }
     catch Exception::AUTH_REQUIRED with {
         my $x = shift;
-        $self->debug(1, 'caught AUTH_REQUIRED exception: ' . $x->text);
+        $self->debug(1, 'caught AUTH_REQUIRED exception: ' . $x->text, 'handler');
         my $view = $x->view || ($self->session ? 'denied' : 'login');
         $self->return_error( $view, $x );
     }
     catch Exception::SERVER_ERROR with {
         my $x = shift;
-        $self->debug(1, 'caught SERVER_ERROR exception: ' . $x->text);
+        $self->debug(1, 'caught SERVER_ERROR exception: ' . $x->text, 'handler');
         $x->log_error;
         $x->notify_admin;
         my $view = $x->view || 'error';
@@ -144,12 +142,12 @@ sub build_page {
     }
     catch Exception::REDIRECT with {
         my $x = shift;
-        $self->debug(1, 'caught REDIRECT exception: ' . $x->text);
+        $self->debug(1, 'caught REDIRECT exception: ' . $x->text, 'handler');
         $self->redirect( $x->redirect_to );
     }
     otherwise {
         my $x = shift;
-        $self->debug(1, 'caught unknown exception: ' . $x->text);
+        $self->debug(1, 'caught unknown exception: ' . $x->text, 'handler');
         $self->return_error( 'error', $x );
     };
     #return $return_code;
@@ -169,7 +167,7 @@ And each step is described below.
 =cut
 
 sub task_sequence {
-    return qw( check_permission adjust_input do_op return_output );
+    return qw( check_permission read_input do_op return_output );
 }
 
 =head3 check_permission() 
@@ -180,70 +178,146 @@ This is just a placeholder, and always returns true. It is very likely that your
 
 sub check_permission { 1 };
 
-=head3 adjust_input() 
+=head3 read_input() 
 
-Placed here as a convenience in case subclasses want to test or adjust the input set. One common tweak is to read path_info. Any changes you make here should be by way of C<set_param>: if you call moniker or id directly, for example, later steps may override your changes.
+Most important variables are retrieved from and stored in the input set. Calling moniker() will get or set the value of the moniker intput parameter, for example.
 
-NB. Most important variables are retrieved from the input set by the corresponding method (eg calling ->moniker) will look at the 'moniker' parameter if it finds no other value to return.
+So this method tries to populate those parameter slots *if they're not already filled*. 
+
+I've recently separated out the ways it tries to do that, so that they can be selectively overridden. The methods that do the work are:
+
+=head3 find_moniker() 
+
+Looks for input in the form moniker=id or moniker=all, by scanning the input parameters for monikers that the factory recognises. 
+
+=head3 translate_path_info() 
+
+Then it will examine the path information, looking for /moniker/id or /moniker/all. If it finds neither, but there is path info, we assume that the path info is a view name. This allows urls like /browse/welcome.html to pick out views.
+
+Any suffix is removed from the path info, so you can put arbitrary .somethings on the end of your urls if you like. The removed suffix can be retrieved by calling handler->suffix if you need it.
 
 =cut
 
-sub adjust_input { 
+sub read_input { 
 	my $self = shift;
-    $self->set_param( moniker => $self->param('type')) if $self->param('type') && ! $self->param('moniker');
+    $self->debug(3, "CDFH->read_input", 'handler', 'input');
 
-	if ($self->param && ! ($self->param('id') || $self->param('moniker'))) {
-    	$self->debug(3, "input but no id or moniker parameters. looking for class monikers.");
-    	my $id;
-        my $moniker = first { $id = $self->param($_) } @{ $self->factory->classes };
-        if ($moniker) {
-            $self->debug(3, "found a moniker parameter: $moniker. value is $id.");
-            $self->set_param(moniker => $moniker);
-            $self->set_param(id => $id);
-            $self->delete_param($moniker);
-        }
+	$self->find_moniker;
+	$self->translate_path_info;
+    $self->adjust_input;
+}
+
+sub find_moniker {
+	my $self = shift;
+    return if $self->param('id') || $self->param('moniker');
+    return unless $self->param;
+
+	$self->debug(3, "CDFH->find_moniker()", 'handler', 'input');
+	my @monikers = @{ $self->factory->classes };
+	my $moniker = first { $self->param($_) eq 'new' } @monikers;
+	$moniker ||= first { defined $self->param($_) } @monikers;
+			   
+	if ($moniker) {
+		$self->debug(3, "found a moniker parameter: $moniker.", 'handler', 'input');
+		$self->set_param(moniker => $moniker);
+		my @ids = $self->param($moniker);
+		$self->set_param(id => ($#ids) ? \@ids : $ids[0]);
 	}
+}
 
-	if ($self->path_info && ! ( $self->param('id') || $self->param('moniker') || $self->param('view') )) {
-    	$self->debug(3, "path info, but no id, moniker or view parameter. splitting pi.");
-        my ($general, $specific) = $self->read_path_info;
-        if ($general eq 'op' && $specific) {
-            $self->set_param('op', $specific);
-        } elsif ($self->factory->has_class($general)) {
-            $self->set_param('moniker', $general);
-            $self->set_param('id', $specific) unless $general eq 'all';
-        } elsif ($general) {
-            $self->set_param('view', $general);
-        }
-    }
+sub translate_path_info {
+	my $self = shift;
+    return if $self->param('id') || $self->param('moniker');
+    return unless $self->path_info;
+
+	$self->debug(3, "CDFH->translate_path_info().", 'handler', 'input');
+	my ($general, $specific) = $self->read_path_info;
+
+	if ($self->factory->has_class($general)) {
+		$self->set_param('moniker', $general);
+		$self->set_param('id', $specific) unless $general eq 'all';
+	} elsif ($general) {
+		$self->set_param('view', $general);
+	}
+}
+
+=head3 adjust_input( )
+
+Placeholder for a sub that will change input values immediately after read_input has populated them. 
+
+This version just turns type parameters into moniker parameters to paper over an old syntax change (when monikers were first introduced to cdbi).
+
+adjust_input() differs from find_moniker() and translate_path_info() in that it is meant to revise the input set once it has been discovered, rather than helping with its discovery. It shouldn't care how the parameter was supplied.
+
+=cut
+
+sub adjust_input {
+    my $self = shift;
+    $self->debug(3, "CDFH->adjust_input", 'handler', 'input');
+    $self->set_param( moniker => $self->param('type')) if $self->param('type') && ! $self->param('moniker');
 }
 
 =head3 view( view_name )
 
-Looks for a 'view' parameter in input and calls C<permitted_view> to compare it against the configured list of permitted views. 
+Most applications based on CDF will have some sort of view mechanism that will determine what template is used to display the objects or lists requested. This method just retrieves the view parameter and calls check_view.
 
-Can be supplied with a value. Defined but non-true values will be accepted and retained, so to clear the view setting, just call C<view(0)>.
-
-Either way, if a view value is present (and no object is returned by self->thing, in which case the view parameter points to an object template not a proper view), this method throws a NOT_FOUND exception unless it is allowed.
+None of this (simple) machinery is invoked unless a template asks for a view, so you can ignore it if it's not relevant to your application.
 
 =cut
 
 sub view {
 	my $self = shift;
-	$self->debug(4, 'CDFH->view(' . join(',',@_) . ')');
+	$self->debug(4, 'CDFH->view(' . join(',',@_) . ')', 'handler');
     $self->set_param(view => $_[0]) if @_;
-    return unless $self->param('view');
-  	$self->check_view unless $self->thing;
-    return scalar( $self->param('view') );
+    my $view = $self->param('view');
+    return $self->prefer_view unless $view;
+   	throw Exception::NOT_FOUND(-text => "There is no $view view here") unless $self->check_view($view);
+    return $view;
 }
 
-=head3 check_view( view_name )
+=head3 viewdir( view_name )
 
-Placeholder in case your subclass wants to restrict access to some views.
+Returns the name of the directory in which we should look for view templates. Sometimes a subclass likes to change this, usually just for filing reasons.
 
 =cut
 
-sub check_view { 1 }
+sub viewdir { 'views' }
+
+=head3 check_view( view_name )
+
+This is a simple method that checks the supplied view parameter against a list of permitted views (via the permitted_view configuration parameter). It returns true if the view is permitted, false if not. It is very likely that you will override this method in subclass to add session-based permissions.
+
+=cut
+
+sub check_view { 
+    my ($self, $view) = @_;
+    return unless $view;
+    my %permitted_view = map { $_ => 1 } $self->config->get('permitted_view');
+    return $permitted_view{$view};
+}
+
+=head3 default_view( view_name )
+
+Returns the name of the view we should default to if no other instruction is found. This is a relatively low-key instructions that will not be followed until aother avenues have been exhausted: it's normally used by templates looking for something to do. Defaults to the 'default_view' configuration parameter then 'welcome'.
+
+Note that this often means that the usual view-permission check is not applied to the default view.
+
+=cut
+
+sub default_view {
+	my $self = shift;
+    return $self->config->default_view || 'welcome';    
+}
+
+=head3 prefer_view( view_name )
+
+Returns the name of a view that should be imposed if no view parameter is found in input. This is useful for handler subclasses that perform a narrower range of functions and want to preempt a view-choice mechanism that would otherwise be invoked. Does nothing here.
+
+The preferred view will be subject to the usual permission checks.
+
+=cut
+
+sub prefer_view { undef }
 
 =head3 moniker( $moniker )
 
@@ -257,11 +331,12 @@ NB. the moniker, id, op and view parameters are held as request parameters: they
 
 sub moniker {
 	my $self = shift;
-	$self->debug(4, 'CDFH->moniker(' . join(',',@_) . ')');
+	$self->debug(4, 'CDFH->moniker(' . join(',',@_) . ')', 'handler', 'input');
     $self->set_param(moniker => $_[0]) if @_;
-    return unless $self->param('moniker');
-  	throw Exception::NOT_FOUND(-text => "No '" . $self->param('moniker') . "' data class found") unless $self->check_moniker;
-    return scalar( $self->param('moniker') );
+    my $moniker = scalar($self->param('moniker')) || return;
+  	return unless $self->check_moniker($moniker);
+	$self->debug(4, "moniker is $moniker", 'handler', 'input');
+    return $moniker;
 }
 
 =head3 check_moniker( $moniker )
@@ -272,8 +347,7 @@ Checks that the supplied moniker is among those managed by the local factory. Su
 
 sub check_moniker {
     my ($self, $moniker) = @_;
-    $moniker ||= scalar($self->param('moniker'));
-	$self->debug(4, 'CDFH->check_moniker($moniker)');
+	$self->debug(4, 'CDFH->check_moniker($moniker)', 'handler', 'input');
     return $self->factory->has_class($moniker);
 }
 
@@ -285,7 +359,7 @@ Looks for an 'id' parameter in input. Can be supplied with a value instead.
 
 sub id {
 	my $self = shift;
-	$self->debug(4, 'CDFH->id(' . join(',',@_) . ')');
+	$self->debug(4, 'CDFH->id(' . join(',',@_) . ')', 'handler', 'input');
     $self->set_param(id => $_[0]) if @_;
     return scalar( $self->param('id') );
 }
@@ -302,7 +376,7 @@ Returns immediately if the necessary parameters are not supplied. Throws a NOT_F
 
 sub thing {
 	my $self = shift;
-	$self->debug(3, 'CDFH->thing(' . join(',',@_) . ')');
+	$self->debug(3, 'CDFH->thing(' . join(',',@_) . ')', 'handler', 'input');
 	return $self->{thing} = $_[0] if @_;
 	return $self->{thing} if defined $self->{thing};
 	
@@ -348,19 +422,13 @@ Returns if no moniker parameter is found: the ghost has to have a class to shado
 sub ghost {
 	my $self = shift;
 	return unless $self->moniker;
-
-    $self->debug(3, 'CDFH is making a ghost');
+    $self->debug(3, 'CDFH is making a ghost', 'handler', 'ghost');
 
     my $initial_values = { 
         map { $_ => $self->param($_) }
         grep { $self->param($_) }
         $self->factory->columns($self->moniker, 'All')
     };
-    $initial_values->{id} = 'new';
-    $initial_values->{moniker} = $self->moniker;
-    $initial_values->{date} = $self->factory->now;
-    $initial_values->{person} = $self->session->person if $self->session;
-
  	return $self->factory->ghost_object($self->moniker, $initial_values);
 }
 
@@ -372,7 +440,7 @@ Get or set that, by default, returns the 'op' input parameter.
 
 sub op {
 	my $self = shift;
-	$self->debug(3, 'CDFH->op(' . join(',',@_) . ')');
+	$self->debug(3, 'CDFH->op(' . join(',',@_) . ')', 'handler');
     $self->set_param(op => $_[0]) if @_;
     return $self->param('op');
 }
@@ -392,10 +460,10 @@ will result in a call to something like Class::DBI::Factory::Handler->delete(),
 sub do_op {
 	my $self = shift;
 	my $op = $self->op;
-    $self->debug(2, 'do_op: no op') unless $op;
+    $self->debug(2, 'do_op: no op', 'handler') unless $op;
 	return unless $op;
 	my $permitted = $self->permitted_ops;
-    $self->debug(2, 'Checking permission to ' . $op);
+    $self->debug(2, 'Checking permission to ' . $op, 'handler');
     my $op_call = $permitted->{$op};
    	throw Exception::DECLINED(-text => "operation '$op' is not known") unless $op_call;
     $self->$op_call();
@@ -424,18 +492,13 @@ This base class uses the Template Toolkit: override C<return_output> to use some
 
 sub return_output {
 	my $self = shift;
-	$self->view( $self->default_view ) unless $self->view || $self->thing || $self->moniker;
-	$self->process( $self->container_template, $self->assemble_output );
-}
-
-sub default_view {
-	my $self = shift;
-    return $self->config->default_view || 'welcome';    
+	$self->process( $self->master_template, $self->assemble_output );
 }
 
 sub return_error {
 	my ($self, $error, $x) = @_;
-    $self->debug(3, "*** return_error: $error");
+    warn("*** return_error: $error\n");
+    $self->debug(3, "*** return_error: $error", 'handler');
     my $output = $self->minimal_output;
     my $template = $self->config->get('error_page');
     $output->{error} = $error;
@@ -451,7 +514,7 @@ The variables which will be available to templates are assembled here.
 
 sub assemble_output {
 	my $self = shift;
-	$self->debug(4, '* CDFH: assemble_output');
+	$self->debug(4, '* CDFH: assemble_output', 'handler', 'output');
 	my $extra = $self->extra_output;
 	my $output = { 
 
@@ -462,21 +525,30 @@ sub assemble_output {
         errors => $self->error || undef,
 		deleted_object => $self->deleted_object || undef,
 		report => $self->report || undef,
+        referer => $self->referer || undef,
 
 		view => $self->view || undef,
+		viewdir => $self->viewdir || undef,
+		default_view => $self->default_view || undef,
 		moniker => $self->moniker || undef,
 		id => $self->id || undef,
 		thing => $self->thing || undef,
-		list => $self->list || undef,
 
         %$extra,
     };
+    
+    if ($self->factory->debug_level >= 5) {
+        my $dumper = Data::Dumper->new([$output],['assembled_output']);
+        $dumper->Maxdepth(2);
+        $self->debug(5, $dumper->Dump, 'handler', 'output');
+    }
+    
     return $output;
 }
 
 sub minimal_output {
 	my $self = shift;
-	$self->debug(4, 'minimal_output)');
+	$self->debug(4, 'minimal_output)', 'handler', 'output');
     my $output = {
 		handler => $self,
 		factory => $self->factory,
@@ -496,46 +568,10 @@ This is called by assemble_output, and the hashref it returns is appended to the
 
 sub extra_output {
 	my $self = shift;
-	$self->debug(4, '* CDFH: extra_output');
+	$self->debug(4, '* CDFH: extra_output', 'handler', 'output');
 	return {};
 }
 
-=head3 pager( ignore_id )
-
-If a moniker parameter has been supplied, and corresponds to a valid data class, this method will return a pager object attached to that class. If there's a page parameter, that will be passed on too.
-
-Normally this method will return undef if an id parameter is also supplied, assuming that an object rather than a pager is required. Supply a true value as the first parameter and this reluctance will be overridden.
-
-=cut
-
-sub pager {
-	my ($self, $insist) = @_;
-	return if $self->id && ! $insist;
-	return unless $self->moniker;
-    $self->{pager} = $self->factory->pager($self->moniker, $self->param('page'));
-    @{ $self->{contents} } = $self->{pager}->retrieve_all();
-    return $self->{pager};
-}
-
-=head3 list( list_object )
-
-If a moniker parameter has been supplied, this will return an object of Class::DBI::Factory::List attached to the corresponding data class. 
-
-Any other parameters that match columns of the data class will also be passed through, along with any of the list-control flags (sortby, sortorder, startat and step).
-
-As with pager, if there is an id parameter then the list will only be built if you pass a true value to the method.
-
-=cut
-
-sub list {
-	my ($self, $insist) = @_;
-	return if $self->id && ! $insist;
-	return unless $self->moniker;
-    my %list_criteria = map { $_ => scalar( $self->param($_) ) } grep { $self->param($_) } $self->factory->columns($self->moniker, 'All');
-    $list_criteria{$_} = $self->param($_) for grep { $self->param($_) } qw( sortby sortorder startat step );
-    return $self->{list} = $self->factory->list( $self->moniker, %list_criteria );
-}
-
 =head3 session( )
 
 This is just a placeholder, and doesn't do or return anything. It is included in the default set, on the assumption that the first thing you do will be to supply a session-handling mechanism: all you have to do is override this session() method.
@@ -554,29 +590,21 @@ This value is passed to the Template Toolkit along with the bundle of value retu
 
 =cut
 
-sub container_template {
+sub master_template {
 	my $self = shift;
-    return $self->{_container_template} = $_[0] if @_;
-    return $self->{_container_template} if $self->{_container_template};
-    return $self->{_container_template} = $self->default_template;
-}
-
-sub default_template {
-	my $self = shift;
-    return $self->config->get('default_template');
+    return $self->{_template} = $_[0] if @_;
+    return $self->{_template} ||= $self->config->get('master_template');
 }
 
 =head1 BASIC OPERATIONS
 
 This small set of methods provides for the most obvious operations performed on cdbi objects: create, update and delete. Most of the actual work is delegated to factory methods.
 
-A real application will also include non-object related operations like logging in and out, registering and making changes to sets or classes all at once.
+A real application will replace these with something less crappy and also include non-object related operations like logging in and out, registering and making changes to sets or classes all at once.
 
 =head2 store_object()
 
-Uses the input set to create or update an object.
-
-The resulting object is stored in $self->thing.
+Uses the input set to create or update the object already held as $self->{thing}.
 
 =head2 delete_object()
 
@@ -586,9 +614,9 @@ calls delete() on the foreground object, but first creates a ghost copy and stor
 
 sub store_object {
 	my $self = shift;
-	$self->debug(1, "CDFH: store_object");
+	$self->debug(1, "CDFH: store_object", 'handler', 'storage');
 	return unless $self->thing;
-	$self->debug(1, "thing is " . $self->thing->title);
+	$self->debug(1, "thing is " . $self->thing->title, 'handler', 'storage');
 
     # if this is a new object then thing() will return a ghost that just needs to be solidified with make().
 
@@ -624,20 +652,18 @@ sub deleted_object {
 
 $handler->factory->retrieve_all('artist');
 
-Returns the local factory object, or creates one if none exists yet. You can also pass in a factory object, though I can't imagine many cirumstances where this would be required. I only use during the installation tests.
+Returns the local factory object, which will create it if it doesn't exist yet. 
 
 =head2 factory_class()
 
-returns the full name of the class that should be used to instantiate the factory. Defaults to Class:DBI::Factory, of course: if you subclass the factory class, you must mention the name of the subclass here.
+Returns the full name of the class that should be used to instantiate the factory. Defaults to Class:DBI::Factory, of course: if you subclass the factory class, you must mention the name of the subclass here.
 
 =cut
 
 sub factory_class { "Class::DBI::Factory" }
-
 sub factory { 
-    my $self = shift;
-    return $self->{_factory} = $_[0] if @_;
-    return $self->{_factory} ||= $self->factory_class->instance(); 
+	my $self = shift;
+	return $self->{_factory} ||= $self->factory_class->instance; 
 }
 
 =head2 request()
@@ -689,7 +715,7 @@ Accepts a (fully specified) template address and output hashref and passes them
 
 sub process {
 	my ($self, $template, $output) = @_;
-	$self->debug(3, "DMH: processing template: '$template'");
+	$self->debug(3, "DMH: processing template: '$template'", 'handler', 'template');
 	$self->send_header;
     $self->factory->process($template, $output, $self->request);
 }
@@ -705,7 +731,7 @@ Any supplied values are assumed to be messages for the user, and pushed onto an
 
 sub report {
 	my $self = shift;
-	$self->debug(2, @_);
+	$self->debug(2, $_, 'handler') for @_;
     push @{ $self->{_report} }, @_;
     return $self->{_report};
 }
@@ -723,7 +749,7 @@ sub error {
 	my $self = shift;
 	my @errors = @_;
 	$self->{_errors} ||= [];
-	$self->debug(1, 'error messages: ' . join('. ', @errors));
+	$self->debug(1, 'error messages: ' . join('. ', @errors), 'handler');
     push @{ $self->{_errors} }, @_;
     return $self->{_errors};
 }
@@ -753,7 +779,7 @@ Unlike the factory's debugging methods, these are intended to hold and return me
 sub log {
 	my ($self, @messages) = @_;
 	push @{ $self->{_log} }, @messages;
-	$self->debug(1, @messages);
+	$self->debug(1, "log: $_", 'handler') for @messages;
 }
 
 sub report {
@@ -779,18 +805,17 @@ Returns the full address of this request (ie url?qs)
 
 sub url {
 	my $self = shift;
-	return $self->request->uri;    # changed for mod_perl2 from url to url
+	return $self->request->uri;
 }
 
 sub full_url {
 	my $self = shift;
-	return $self->url . "?"; # . $self->qs;        need to work out how to do this under mod_perl2
+	return $self->url . '?' . $self->qs;    # changed for mod_perl2
 }
 
 sub qs {
 	my $self = shift;
-	my $r = $self->request;
-	my $qs = $r->args; # updated for mod_perl2, since Apache::RequestRec has no query_string method. docs are oddly silent on this.
+	my $qs = $self->request->query_string;
 	return $qs;
 }
 
@@ -807,7 +832,7 @@ then the path_info will be /bar/kettle/black. Note that the opening / will cause
 sub path_info {
 	my $self = shift;
 	my $pi = $self->request->path_info;
-	$self->debug(3, "path_info is $pi");
+	$self->debug(3, "*** path_info is $pi", 'handler', 'input');
 	return $pi;
 }
 
@@ -912,9 +937,10 @@ Erases all input by calling delete_param() for all input parameters.
 
 sub param {
 	my ($self, $p) = @_;
-	$self->debug(5, "CDFH->param($p);");
+	$self->debug(5, "CDFH->param($p);", 'handler', 'input');
 	return $self->request->param unless $p;
 	my @input = map { $self->factory->inflate_if_possible($p => $_) } $self->request->param($p);
+	return $input[0] unless $#input;
 	return wantarray ? @input : $input[0];
 }
 
@@ -927,10 +953,10 @@ sub has_param {
 
 sub all_param {
 	my $self = shift;
-    $self->debug(3, 'CDFH: all_param');
+    $self->debug(3, 'CDFH: all_param', 'handler', 'input');
 	my %p;
 	my @param_names = $self->request->param;
-    $self->debug(4, "input parameters: " . join(', ', @param_names));
+    $self->debug(4, "input parameters: " . join(', ', @param_names), 'handler', 'input');
 	for ( @param_names ){
 	    my @input = $self->param($_);
 	    $p{$_} = ($#input) ? \@input : $input[0];
@@ -939,18 +965,14 @@ sub all_param {
 }
 
 sub set_param {
-	my ($self, $param, $value) = @_;
-	$self->debug(5, "CDFH->set_param($param => $value);");
-    return unless $param;
-	$self->request->args->{$param} = "$value";   
-
-	# updated for mod_perl2. 
-	# NB. Apache::RequestRec really hates having cdbi objects in the args table. segfaults every time. It'll be reinflated on a call to param($param)
+	my $self = shift;
+	$self->debug(4, 'set_param(' . join(',',@_) . ')');
+	return $self->request->param(@_);
 }
 
 sub delete_param {
 	my $self = shift;
-	$self->debug(4, 'delete_param(' . join(',',@_) . ')');
+	$self->debug(4, 'delete_param(' . join(',',@_) . ')', 'handler', 'input');
 	my $table = $self->request->param; # small change for mod_perl2: parms method now deprecated
 	$table->unset($_) for @_;
 }
@@ -1022,21 +1044,33 @@ Under mod_perl2 all this has to do is set the content type, which it does by cal
 
 =head2 mime_type( $type )
 
-A simple method that can be used to get or set the mime-type for this response, or be subclassed to make some other decision altogether. Defaults to:
+Simple mutator that will get or set the mime-type for this response, or can be subclassed to make some other decision altogether. Defaults to:
 
 =head2 default_mime_type()
 
-Returns the mime type that will be used if no other is specified. The default default is text/html: override in subclass if that doesn't suit.
+Returns the mime type that will be used if no other is specified. The default default is text/html: set a default_mime_type parameter or subclass the method to do something more complicated.
 
 =cut
 
 sub send_header {
 	my $self = shift;
+	return if $self->{_header_sent};
 	$self->request->content_type($self->mime_type);
+	$self->request->no_cache(1) if $self->no_cache;	
+	$_->bake for @{ $self->{cookies_out} };
+	$self->request->send_http_header;
+    $self->header_sent(1);
 }
 
+sub no_cache { 0 };
 sub default_mime_type { 'text/html' }
 
+sub header_sent {
+	my $self = shift;
+    return $self->{_header_sent} = $_[0] if @_;
+    return $self->{_header_sent};
+}
+
 sub mime_type {
 	my $self = shift;
     return $self->{_mime_type} = $_[0] if @_;
@@ -1044,6 +1078,11 @@ sub mime_type {
     return $self->{_mime_type} = $self->default_mime_type;
 }
 
+sub default_mime_type { 
+	my $self = shift;
+    return $self->config->default_mime_type || 'text/html';    
+}
+
 =head2 set_cookie( hashref )
 
 $handler->set_cookie({
@@ -1063,13 +1102,11 @@ sub set_cookie {
         $self->debug(1, 'set_cookie: headers already sent');
         return;
     }
-    for (@_) {
-        my $cookie = Apache::Cookie->new($self->request, %{ $_ });
-        $cookie->bake;
-    }
+	push @{ $self->{cookies_out} }, map { Apache::Cookie->new($self->request, %{ $_ }) } @_;
 	return 1;
 }
 
+
 =head2 redirect( full_url )
 
 $handler->redirect('http://www.spanner.org/cdf/')
@@ -1088,6 +1125,7 @@ sub redirect {
     }
 	my $url = shift || $self->{redirect} || $self->factory->config('url');
     $self->debug(3, "*** redirect: bouncing to $url");
+	$self->request->err_headers_out->add('Set-Cookie' => $_) for @{ $self->{cookies_out} };
 	$self->request->err_headers_out->add( Location => $url );
 	return REDIRECT;
 }
@@ -1102,7 +1140,7 @@ This is normally called from an exception handler: the task sequence is stopped
 
 sub redirect_to_view {
 	my ($self, $view) = @_;
-    $self->debug(3, "*** redirect_to_view: bouncing to view $view");
+    $self->debug(3, "*** redirect_to_view: bouncing to view $view", 'handler');
     $self->view( $view );
     return $self->return_output;
 }
