@@ -11,7 +11,4 @@ sub class_title { 'Thing' }
 sub class_plural { 'Things' }
 sub class_description { 'Things of one kind or another.' }
 
-sub factory { return Class::DBI::Factory->instance; }
-sub db_Main { return shift->factory->dbh; }
-
 1;
